﻿using Styx;
using Styx.Common;
using Styx.Common.Helpers;
using Styx.Patchables;
using Styx.Plugins;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Media;

namespace Jim87ObjectsFinder
{
    public class Jim87ObjectsFinder : HBPlugin
    {
        private ArrayList sentObjs = new ArrayList();

        public override string Author
        {
            get { return "jim87"; }
        }

        public override string Name
        {
            get { return "Jim87's Objects detector & Navigator"; }
        }

        public override Version Version
        {
            get { return new Version(1, 2, 11); }
        }

        public override bool WantButton
        {
            get
            {
                return true;
            }
        }

        public override string ButtonText
        {
            get
            {
                return "Where am I?";
            }
        }

        private readonly WaitTimer clock = new WaitTimer(TimeSpan.FromMilliseconds(1500));

        private WoWPoint objPosition = WoWPoint.Empty,
                        zoneStart = WoWPoint.Empty,
                        zoneEnd = WoWPoint.Empty;
        List<uint> toFind = new List<uint>();

        public override void OnButtonPress()
        {
            SoundPlayer player = new SoundPlayer(@"Plugins/OnyxEggsDetector/sonar.wav");
            player.Play();

            uint zoneID = Lua.GetReturnVal<uint>("return GetCurrentMapAreaID()", 0);

            WoWPoint actual = PolarToLocalCoords(StyxWoW.Me.GetPosition(), zoneID);
            Logging.Write("{0} {1}", StyxWoW.Me.GetPosition().Y, StyxWoW.Me.GetPosition().X);
            Logging.Write("[{0}, {1}]", actual.X, actual.Y);
        }

        public override void Initialize()
        {
            Logging.Write(Name + " v." + Version.ToString());
            toFind.Add(214945); // Onyx Egg
            toFind.Add(210565); // Dark Soil
            toFind.Add(185915); // Netherwing Egg
        }

        public override void Pulse() {
            if (!clock.IsFinished) return;

            clock.Reset();

            var objects = ObjectManager.GetObjectsOfType<WoWGameObject>(true, true).Where(o => toFind.Contains(o.Entry));
            
            foreach (var obj in objects)
            {
                if (sentObjs.Contains(obj.Guid)) continue;

                uint zoneID = Lua.GetReturnVal<uint>("return GetCurrentMapAreaID()", 0);

                objPosition = PolarToLocalCoords(obj.GetPosition(), zoneID);

                Logging.Write(obj.Name + " found at [" + objPosition.X + " " + objPosition.Y + "]");
                Lua.DoString("TomTom:AddWaypoint(" + objPosition.X + ", " + objPosition.Y + ", \"Yummy " + obj.Name + "!\")");

                SoundPlayer player = new SoundPlayer(@"Plugins/Jim87ObjectsFinder/sonar.wav");
                player.Play();
                sentObjs.Add(obj.Guid);

                // untill proper solution
                Styx.CommonBot.TreeRoot.Stop();
            }
        }

        public static WoWPoint PolarToLocalCoords(WoWPoint polarCoords, uint zoneID)
        {
            WoWPoint zoneStart = new WoWPoint(), zoneEnd = new WoWPoint();

            WoWDb.DbTable WorldMapArea = StyxWoW.Db[ClientDb.WorldMapArea];
            WoWDb.Row zone = WorldMapArea.GetRow(zoneID);

            zoneStart.Y = zone.GetField<float>(4);
            zoneEnd.Y = zone.GetField<float>(5);
            zoneStart.X = zone.GetField<float>(6);
            zoneEnd.X = zone.GetField<float>(7);

            WoWPoint retVal;
            retVal.Z = 0;
            
            float x_weight = (Math.Abs(zoneStart.X - zoneEnd.X)) / 100;
            float y_weight = (Math.Abs(zoneStart.Y - zoneEnd.Y)) / 100;

            // Normalize coords
            polarCoords.X = (float)Math.Round((Math.Max(zoneStart.X, zoneEnd.X) - polarCoords.X) / x_weight, 2);
            polarCoords.Y = (float)Math.Round((Math.Max(zoneStart.Y, zoneEnd.Y) - polarCoords.Y) / y_weight, 2);

            retVal.X = polarCoords.Y;
            retVal.Y = polarCoords.X;

            return retVal;
        }
    }
}
